
package bg.unisofia.fmi.grosd.ns.peristent_task;

import java.io.Serializable;

import bg.unisofia.fmi.grosd.ns.exception.NSClassLoaderException;
import bg.unisofia.fmi.grosd.ns.processor.NSTaskProcessor;
import bg.unisofia.fmi.grosd.ns.remote.NodeServiceClientCallback;
import bg.unisofia.fmi.grosd.ns.remote.task.NSJavaPersistentTaskInvocation;


/**
 * Major interface for representing and storing Java persistent tasks on this node.
 * This is an internal representation of the persistent tasks received from the API. 
 *@author Shenol Yousouf
 */
public interface NSJavaPersistentTask extends Serializable {

  /**
   * Process an invocation request.
   * @param arg <code>NSJavaPersistentTaskInvocation</code> request
   * @param processor managing <code>NSTaskProcessor</code> instance used for tracking the execution progress
   * @param client an object for callback with the Node Service API
   */
  public void process (NSJavaPersistentTaskInvocation arg, NSTaskProcessor processor, NodeServiceClientCallback client);
  
  /**
   * Stop current task executed by the persistent service.
   * @param synchronous for synchronous or asynchronous execution of stopping procedure
   */
  public void stopTask(boolean synchronous);
  
  /**
   * Stop the service of the persistent task.
   * @param forcedMode if <code>true</code>, the service will be stopped even in the middle of executing a request;
   * if <code>false</code>, the service will be stopped after the end of execution
   */
  public void stopService(boolean forcedMode);
  
  /**
   * 
   * @return if the persistent service is busy with the execution of a task which means that it cannot handle another task
   */
  public boolean isBusy();
  
  /**
   * 
   * @return if the persistent service is stopped; while stopped, all invocation requests are rejected
   */
  public boolean isStopped();
  
  /**
   * Set stopped state of persistent task.
   * @param value if <code>true</code>, the persistent service is in stopped state; if <code>false</code>,
   * it can accept invocation requests.
   */
  public void setStopped(boolean value);
  
  /**
   * 
   * @return key identifier of the persistent service to which the requests are addressed inside the node
   */
  public String getKeyID();
  
  /**
   * 
   * @return identifier of the currently executed task
   */
  public String getCurrentTaskID();
  
  /**
   * Load classes necessary for the work of the persistent service.
   * @throws NSClassLoaderException if an error occurs during classloading operations
   */
  public void loadServiceClasses() throws NSClassLoaderException;
}
