
package bg.unisofia.fmi.grosd.ns.peristent_task.impl;

import java.io.File;
import java.rmi.RemoteException;
import java.util.HashSet;
import java.util.Set;

import bg.unisofia.fmi.grosd.ns.exception.NSInitializationException;
import bg.unisofia.fmi.grosd.ns.peristent_task.NSJavaPersistentTask;
import bg.unisofia.fmi.grosd.ns.peristent_task.impl.serializables.NSJavaPersistentTaskSerializablesFactory;
import bg.unisofia.fmi.grosd.ns.peristent_task.impl.serializables.NSTaskMethodPrototypeSerializable;
import bg.unisofia.fmi.grosd.ns.peristent_task.impl.serializables.NSTaskTargetClassSerializable;
import bg.unisofia.fmi.grosd.ns.remote.NodeServiceClientCallback;
import bg.unisofia.fmi.grosd.ns.remote.task.NSJavaPersistentTaskUpload;
import bg.unisofia.fmi.grosd.ns.remote.task.arg.FileArgument;
import bg.unisofia.fmi.grosd.ns.remote.task.method.NSTaskMethodPrototype;
import bg.unisofia.fmi.grosd.ns.remote.task.targetclass.NSTaskTargetClass;
import bg.unisofia.fmi.grosd.ns.util.NSFileTranferUtilities;
import bg.unisofia.fmi.grosd.ns.util.NodeServiceUtilities;


/**
 * Factory for creating <code>NSJavaPersistentTask</code> insatnces.
 *@author Shenol Yousouf
 */
public abstract class NSJavaPersistentTaskFactory {

  /**
   * 
   * @param remote request for persistent task upload, received from the API
   * @param client object for callback connection with the API
   * @param filesForGC list files for removal together with the persistent task
   * @return a persistent task instance
   * @throws NSInitializationException
   */
  public static NSJavaPersistentTask buildNSPersistentTask(NSJavaPersistentTaskUpload remote, NodeServiceClientCallback client, Set<File> filesForGC) throws NSInitializationException {
    try {
      String keyID = remote.getKeyIdentifier();
      File workFolder = NodeServiceUtilities.getWorkFolderForPersistentTask(keyID);
      if (workFolder.exists() == false && workFolder.mkdirs() == false) {
        throw new NSInitializationException("Cannot create work folder for service " + keyID + " - " + workFolder.getAbsolutePath());
      }
      
      FileArgument[] files = remote.getBinaryFiles();
      Set<File> binariesList = new HashSet<File>(files.length);
      NSFileTranferUtilities.transferClientFiles(binariesList, client, workFolder, files, true);
      File[] binaries = new File[binariesList.size()];
      binariesList.toArray(binaries);
      filesForGC.add(workFolder);
      
      NSTaskTargetClass targetClassRemote = remote.getTargetClass();
      NSTaskTargetClassSerializable targetClass = NSJavaPersistentTaskSerializablesFactory.buildNSTaskTargetClassSerializable(targetClassRemote, client, filesForGC);
      
      NSTaskMethodPrototypeSerializable method = null;
      NSTaskMethodPrototype methodRemote = remote.getDefaultMethodPrototype();
      if (methodRemote != null) {
        method = NSJavaPersistentTaskSerializablesFactory.buildNSTaskMethodPrototypeSerializable(targetClass.getClassName(), methodRemote);
      }
      
      return new NSJavaPersistentTaskImpl(keyID, binaries, method, targetClass);
      
    } catch (RemoteException e) {
      // TODO: log error
      e.printStackTrace();
      throw new NSInitializationException("Error creating instrance of " + NSJavaPersistentTask.class.getName() + " from remote instance of " + remote.getClass().getName(), e);
    }  
    
  }
}
