
package bg.unisofia.fmi.grosd.ns.processor;

import bg.unisofia.fmi.grosd.ns.exception.NodeServiceException;


/**
 * General-purpose processing unit of the Node Service kernel.
 * Its various types and implementations specialize in processing the different task types.
 * All processor types run in threads during execution so that they are decoupled from the main dispatcher of the  Node Service kernel. 
 *@author Shenol Yousouf
 */
public interface NSTaskProcessor extends Runnable{
  
 /**
  * Stop task execution if possible.
   * @param synchronous indicates if the method ends synchronously with the actual task termination
   * @throws <code>NodeServiceException</code> if the stop operation is not supported by this processor
   */
  public void stopTask(boolean synchronous) throws NodeServiceException;
  
  /**
   * 
   * @return key identifier of the currently executed task
   */
  public String getTaskID();
}
