
package bg.unisofia.fmi.grosd.ns.processor.impl;

import static bg.unisofia.fmi.grosd.ns.enums.NSTaskActionState.ABORTED;

import java.rmi.RemoteException;

import bg.unisofia.fmi.grosd.ns.exception.NodeServiceException;
import bg.unisofia.fmi.grosd.ns.peristent_task.NSJavaPersistentTask;
import bg.unisofia.fmi.grosd.ns.processor.NSTaskProcessor;
import bg.unisofia.fmi.grosd.ns.remote.NodeServiceClientCallback;
import bg.unisofia.fmi.grosd.ns.remote.task.NSJavaPersistentTaskInvocation;
import bg.unisofia.fmi.grosd.ns.repository.NSTaskRepository;
import bg.unisofia.fmi.grosd.ns.util.NSTaskLifecycleNotifier;


/**
 * Processor unit for invoking a specified persistent task.
 *@author Shenol Yousouf
 */
public class NSJavaPersistentTaskInvocationProcessor extends Thread implements NSTaskProcessor {
  
  private NSJavaPersistentTaskInvocation task;
  private String taskID;
  private String keyID; 
  private NodeServiceClientCallback client;
  
  private NSJavaPersistentTask executor;

  public NSJavaPersistentTaskInvocationProcessor (NSJavaPersistentTaskInvocation task, NodeServiceClientCallback client) throws RemoteException {
    this.task = task;
    this.taskID = task.getTaskID();
    this.keyID = task.getKeyIdentifier();
    this.client = client;
  }
  
  public void run() {
    // do initial pre-processing for identifying the responsible executor
    executor = NSTaskRepository.getServiceRecord(keyID).getProcessor();
    
    if (executor == null) {
      notifyTaskAborted("No processor found for key " + keyID);
    } else {
      while(executor.isBusy()) {
        // in current implementation, processor waits for executor to finish its job;
        // in alternative version, it can throw an exception that it cannot invoke task right now
        try {
          wait();
        } catch (InterruptedException e) {
          // TODO: log exception
          e.printStackTrace();
        }
      }
      
      // executor is free now so we get it started
      executor.process(task,this, client);
    }
  }
  
  private void notifyTaskAborted(String msg)  {
        
    try {
      task.changeActionState(ABORTED, "Task with ID " + taskID + " has been aborted. Reason: " + msg);
      NSTaskLifecycleNotifier.notifyTaskAborted(task, "Task with ID " + taskID + "  aborted on start - " + msg, new Exception[] {});
    } catch (Exception e) {
      // TODO: log  error
      e.printStackTrace();
    }
  }
  
  public void stopTask(boolean synchronous) throws NodeServiceException {
    if (executor == null) {
      // TODO: log fatal error
      throw new NodeServiceException("No executor is assigned to the current processor so there is no operation to stop.");
    }
    
    if (taskID.equals(executor.getCurrentTaskID()) == false) {
      // executor is performing another task now
      throw new NodeServiceException("Task with ID " + taskID + " is not being executed on node now.");
    }

    executor.stopTask(synchronous);
  }

  public String getTaskID() {
    return taskID;
  }

}
