
package bg.unisofia.fmi.grosd.ns.processor.visitor;

import java.io.File;
import java.io.Serializable;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Set;

import bg.unisofia.fmi.grosd.ns.exception.NodeServiceException;
import bg.unisofia.fmi.grosd.ns.remote.NodeServiceClientCallback;
import bg.unisofia.fmi.grosd.ns.remote.task.arg.FileArgument;
import bg.unisofia.fmi.grosd.ns.remote.task.arg.SerializableArgument;
import bg.unisofia.fmi.grosd.ns.remote.task.arg.visitor.NSTaskArgumentVisitor;
import bg.unisofia.fmi.grosd.ns.util.NSClassChecker;
import bg.unisofia.fmi.grosd.ns.util.NSFileTranferUtilities;


/**
 * Visitor for processing the <code>NSTaskArgument</code> instances.
 * This visitor is responsible for loading the arguments classes during  processing.
 *@author Shenol Yousouf
 */
public class NSMethodArgumentVisitor implements NSTaskArgumentVisitor {
  
  private ArrayList<Class> argClasses;
  private ArrayList<Serializable> argValues;
  
  private Set<File> filesForGC;
  private File folder;
  private NodeServiceClientCallback client;
  private boolean classloadingEnabled;
  
  public NSMethodArgumentVisitor(NodeServiceClientCallback client, Set<File> filesForGC, File folder, int argCount, boolean classloading) {
    this.argClasses = new ArrayList<Class>(argCount);
    this.argValues = new ArrayList<Serializable>(argCount);
    this.filesForGC = filesForGC;
    this.folder = folder;
    this.client = client;
    this.classloadingEnabled = classloading;
  }

  public void visit(SerializableArgument arg) throws RemoteException {
    if (classloadingEnabled) {
      String classname = arg.getFormalParameterClassname();
      Class c = null;
      try {
        c = NSClassChecker.parseClassname(classname);
        argClasses.add(c);
      } catch (ClassNotFoundException e) {
       throw new NodeServiceException("Error loading argument class for name " + classname, e);
      }
    }
    
    argValues.add(arg.getValue());
  }

  public void visit(FileArgument arg) throws RemoteException {
    NSFileTranferUtilities.transferClientFiles(null, client, folder, new FileArgument[] {arg}, false);
    File value = NSFileTranferUtilities.constructDestinationFile(folder, arg);
    
    if (classloadingEnabled) {
      argClasses.add(File.class);
    }
    argValues.add(value);
    //  register file for GC
    filesForGC.add(value);
  }
  
  public Class[] getClasses() {
    return argClasses.toArray(new Class[] {});
  }
  
  public Object[] getArgValues() {
    return argValues.toArray();
  }

}
