
package bg.unisofia.fmi.grosd.ns.processor.visitor;

import java.io.File;
import java.io.Serializable;
import java.rmi.RemoteException;
import java.util.ArrayList;
import java.util.Set;

import bg.unisofia.fmi.grosd.ns.remote.NodeServiceClientCallback;
import bg.unisofia.fmi.grosd.ns.remote.task.arg.FileArgument;
import bg.unisofia.fmi.grosd.ns.remote.task.arg.SerializableArgument;
import bg.unisofia.fmi.grosd.ns.remote.task.arg.visitor.NSTaskArgumentVisitor;
import bg.unisofia.fmi.grosd.ns.util.NSFileTranferUtilities;


/**
 * Visitor for processing the <code>NSTaskArgument</code> instances.
 * This visitor collects the arguments class names during processing. 
 *@author Shenol Yousouf
 */
public class NSMethodArgumentVisitor2 implements NSTaskArgumentVisitor {
  
  private ArrayList<String> argClassnames;
  private ArrayList<Serializable> argValues;
  
  private Set<File> filesForGC;
  private File folder;
  private NodeServiceClientCallback client;
  private boolean collectClassnames;
  
  public NSMethodArgumentVisitor2(NodeServiceClientCallback client, Set<File> filesForGC, File folder, int argCount, boolean collectClassnames) {
    this.argClassnames = new ArrayList<String>(argCount);
    this.argValues = new ArrayList<Serializable>(argCount);
    this.filesForGC = filesForGC;
    this.folder = folder;
    this.client = client;
    this.collectClassnames = collectClassnames;
  }

  public void visit(SerializableArgument arg) throws RemoteException {
    if (collectClassnames) {
      argClassnames.add(arg.getFormalParameterClassname());
    }
    
    argValues.add(arg.getValue());
  }

  public void visit(FileArgument arg) throws RemoteException {
    NSFileTranferUtilities.transferClientFiles(null, client, folder, new FileArgument[] {arg}, false);
    File value = NSFileTranferUtilities.constructDestinationFile(folder, arg);
    
    if (collectClassnames) {
      argClassnames.add(File.class.getName());
    }
    argValues.add(value);
    //  register file for GC
    filesForGC.add(value);
  }
  
  public String[] getClassnames() {
    return argClassnames.toArray(new String[] {});
  }
  
  public Serializable[] getArgValues() {
    return argValues.toArray(new Serializable[] {});
  }

}
