
package bg.unisofia.fmi.grosd.ns.repository;

import java.io.File;
import java.util.HashSet;
import java.util.Set;


/**
 * A basic Node Service repository record which
 * holds information about the task processor and
 * keeps track of the files for removal after task execution, on client's request.
 *@author Shenol Yousouf
 */
public class NSRepositoryRecord<ProcessorType> {

  private ProcessorType processor;
  private Set<File> filesForCG;
  // add other fields when required
  
   NSRepositoryRecord(ProcessorType processor) {
    this.processor = processor;
    this.filesForCG = new HashSet<File>();
  }
  
  public synchronized void addFilesForGC (File...files) {
    
    for (File file : files) {
      filesForCG.add(file);
    }
  }
  
  public synchronized void removeFilesFromRecord (File...files) {    
    for (File file : files) {
      filesForCG.remove(file);
    }
  }
  
  public synchronized File[] getFilesFromRecord() {
    return filesForCG.toArray(new File[] {});
  }

  public ProcessorType getProcessor() {
    return this.processor;
  }

}
