
package bg.unisofia.fmi.grosd.ns.repository;

import bg.unisofia.fmi.grosd.ns.enums.NSTaskActionState;
import bg.unisofia.fmi.grosd.ns.exception.NodeServiceException;
import bg.unisofia.fmi.grosd.ns.processor.NSTaskProcessor;
import bg.unisofia.fmi.grosd.ns.util.NodeServiceUtilities;


/**
 * An extension of <code>NSRepositoryRecord</code>,
 * specialized for tasks with a lifecycle.
 * An additional property, stored in the record, is the current task action state.
 * @see NSRepositoryRecord
 * @see  NSTaskActionState
 *@author Shenol Yousouf
 */
public class NSTaskRepositoryRecord  extends NSRepositoryRecord<NSTaskProcessor> {

  private NSTaskActionState actionState;
  // add other fields when required
  
   NSTaskRepositoryRecord(NSTaskProcessor processor) {
   super(processor);
    this.actionState = NSTaskActionState.WAITING;
  }
  
  public synchronized void setState(NSTaskActionState state) throws NodeServiceException {
    if (NodeServiceUtilities.isActionStateFinal(this.actionState) == false) {
      this.actionState = state;
    } else {
      throw new NodeServiceException("Fatal error in Node Service- Transition from final state " + this.actionState + " to state " + state + " is forbidden.");
    }
  }
  
  public synchronized NSTaskActionState getState() {
    return this.actionState;
  }
  
  // TODO: unfinished implementation
  public synchronized boolean isActionStateFinal() {
    return NodeServiceUtilities.isActionStateFinal(this.actionState);
  }

}
