
package bg.unisofia.fmi.grosd.ns.server;

import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.rmi.server.UnicastRemoteObject;

import bg.unisofia.fmi.grosd.ns.impl.NodeServiceImpl;
import bg.unisofia.fmi.grosd.ns.remote.NodeService;
import static bg.unisofia.fmi.grosd.ns.remote.NodeService.*;


/**
 * The server application for registering the Node Service implementation in the RMI registry of the current host.
 * Thus, the Node Service instance is exposed for direct lookup and usage by the APIs on client hosts.
 * The server start is used for the actual Node Service start and future implementations of the server will be able
 * to stop the service too.
 *@author Shenol Yousouf
 */
public class NSServer {
  
  // only one Node Service is allowed per VM
  private static NodeService nsInstance = null;
  private static int nsPort = -1;

  /**
   * @param args
   * @throws Exception if there is already started Node Service in current VM
   */
  public static void main(String[] args) throws Exception {

    // TODO: currently, the server is delegated only with the start of the Node Service;
    // TODO: for better encapsulation with Node Service class, consider moving the server class to ns.impl package and reducing the visibility of NS stopNodeService method
    // TODO: in future implementations, it should be able also to stop it; the start and stop parameters will be parsed from main's args array
    
    int port;    
    if (args.length == 1) {
      port = Integer.parseInt(args[0]);
    } else {
      port = NodeServicePort;
    }
    
    if (nsInstance == null) {
      nsPort = port;
      
      NodeServiceImpl nsObj = new NodeServiceImpl();
      
      try {
        nsInstance = (NodeService) UnicastRemoteObject.exportObject(nsObj, 0);
        
        Registry registry = LocateRegistry.getRegistry(nsPort);
        registry.rebind(NodeServiceName, nsInstance);
        System.out.println("Node Service server successfully started at port " + NodeServicePort + "...");
      } catch (RemoteException e) {
        // TODO: Better log the message in some server log when it is implemented.
        e.printStackTrace();
      }
    } else {
        throw new Exception("There is already started Node Service on port" + nsPort);
      }
    }

}
