
package bg.unisofia.fmi.grosd.ns.api.impl;

import java.util.HashMap;

import bg.unisofia.fmi.grosd.ns.api.NodeServiceClient;
import bg.unisofia.fmi.grosd.ns.api.exception.CreateNSClientException;
import bg.unisofia.fmi.grosd.ns.remote.NodeService;


/**
 * Factory for creating Node Service clients.
 *@author Shenol Yousouf
 */
public abstract class NSAPIFactory {

  private static HashMap <String, NodeServiceClient> nsClientMappings = new HashMap<String, NodeServiceClient>();
  
  /**
   * @param host node host to which the client has to be connected
   * @return a client of the Node Service kernel which is working on the specified host
   * @throws CreateNSClientException
   */
  public static NodeServiceClient createNodeServiceClient (String host) throws CreateNSClientException {
    return createNodeServiceClient(host, NodeService.NodeServicePort);
  }
  
  /**
   * 
   * @param host node host to which the client has to be connected
   * @param port port on which to establish a connection with the Node Service server
   * @return a client of the Node Service kernel which is working on the specified host and port
   * @throws CreateNSClientException
   */
  public static NodeServiceClient createNodeServiceClient (String host, int port)  throws CreateNSClientException {
    NodeServiceClient result = null;
    String mapKey = getMapKey(host, port);
    result = nsClientMappings.get(mapKey);
    if (result == null) {
      try {
        result = new NodeServiceClientImpl(host, port);
      } catch (Exception ex) {
        throw new CreateNSClientException("Error creating client to the Node Service on host " + host + " and port " + port, ex);
      }
      nsClientMappings.put(mapKey, result);
    }
    return result;
  }
  
  /**
   * Remove a cached client of the Node Service on the specified host and port.
   * @param host
   * @param port
   */
  static void removeNodeServiceClient(String host, int port) {
    nsClientMappings.remove(getMapKey(host, port));
  }
  
  private static String getMapKey (String host, int port) {
    return host + ":" + port;
  }
}
