
package bg.unisofia.fmi.grosd.ns.api.impl;

import java.io.BufferedInputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.net.URISyntaxException;
import java.net.URL;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;

import bg.unisofia.fmi.grosd.ns.remote.NodeServiceClientCallback;
import bg.unisofia.fmi.grosd.ns.remote.util.NSOutputStreamWrapper;
import bg.unisofia.fmi.grosd.ns.remote.util.impl.NSOutputStreamWrapperImpl;


/**
 * @see NodeServiceClientCallback
 *@author Shenol Yousouf
 */
public class NodeServiceClientCallbackImpl implements NodeServiceClientCallback {
  
  final static int BUFFER = 4096;

  public synchronized NSOutputStreamWrapper getOutputStream(String fileName) throws RemoteException {
    try {
      File localFile = new File(fileName);
      File parentFile = localFile.getParentFile();
      if (parentFile != null && !parentFile.exists()) {
        if (!parentFile.mkdirs()) {
          throw new RemoteException("Cannot create containing folders - " + parentFile.getAbsolutePath());
        }
      }
      NSOutputStreamWrapperImpl wrapperImpl = new NSOutputStreamWrapperImpl(localFile);
      NSOutputStreamWrapper wrapper = (NSOutputStreamWrapper) UnicastRemoteObject.exportObject(wrapperImpl, 0);
      return wrapper;
    } catch (IOException e) {
      throw new RemoteException("File " + fileName + " cannot be found", e);
    }
  }

  public synchronized void copyFile(NSOutputStreamWrapper out, URL clientFile) throws RemoteException {
    BufferedInputStream in = null;
    try {
       in = new BufferedInputStream(new FileInputStream(new File(clientFile.toURI())));
    } catch (FileNotFoundException e) {
      throw new RemoteException("File " + clientFile + " cannot be found", e);
    } catch (URISyntaxException e) {
      throw new RemoteException("URL " + clientFile + " cannot be parsed correctly.", e);
    }
    
    byte[] b = new byte[BUFFER];
    int length = 0;
    try {
      while((length = in.read(b)) != -1) {
        out.write(b, length);
      }
    } catch (IOException e) {
      throw new RemoteException("Error copying file from URL " + clientFile + ".", e);
    } finally {
      try {
        in.close();
      } catch (IOException e) {
        // this error is not fatal
        e.printStackTrace();
      }
      out.close();
    }
  }

}
