
package bg.unisofia.fmi.grosd.ns.api.impl;

import static bg.unisofia.fmi.grosd.ns.enums.NodeServiceTaskType.NS_JAVA_GENERIC_TASK;
import static bg.unisofia.fmi.grosd.ns.enums.NodeServiceTaskType.NS_JAVA_PERSISTENT_TASK_INVOCATION;
import static bg.unisofia.fmi.grosd.ns.enums.NodeServiceTaskType.NS_JAVA_PERSISTENT_TASK_UPLOAD;

import java.rmi.NotBoundException;
import java.rmi.RemoteException;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;
import java.rmi.server.UnicastRemoteObject;

import bg.unisofia.fmi.grosd.ns.api.NodeServiceClient;
import bg.unisofia.fmi.grosd.ns.enums.NSTaskActionState;
import bg.unisofia.fmi.grosd.ns.enums.NodeServiceTaskType;
import bg.unisofia.fmi.grosd.ns.remote.NodeService;
import bg.unisofia.fmi.grosd.ns.remote.NodeServiceClientCallback;
import bg.unisofia.fmi.grosd.ns.remote.task.NSJavaPersistentTaskInvocation;
import bg.unisofia.fmi.grosd.ns.remote.task.NSJavaPersistentTaskUpload;
import bg.unisofia.fmi.grosd.ns.remote.task.NSJavaTask;
import bg.unisofia.fmi.grosd.ns.remote.task.NSTaskRemote;


/**
 * @see NodeServiceClient
 *@author Shenol Yousouf
 */
public class NodeServiceClientImpl implements NodeServiceClient {
  
  private NodeService nodeService;
  private String host;
  private int port;
  
  NodeServiceClientImpl(String host) throws RemoteException, NotBoundException {
    this(host, NodeService.NodeServicePort);
  }
  
  NodeServiceClientImpl(String host, int port) throws RemoteException, NotBoundException {
    this.host = host;
    this.port = port;
    Registry registry = LocateRegistry.getRegistry(host, port);
    nodeService = (NodeService) registry.lookup(NodeService.NodeServiceName);
  }

  public synchronized void uploadJavaTask(NSJavaTask task) throws RemoteException {
   /* if (task == null) {
      throw new IllegalArgumentException("Provided task is null.");
    }
    
    NSJavaTask remoteTask = (NSJavaTask) UnicastRemoteObject.exportObject(task, 0);
    
    NodeServiceClientCallbackImpl callback = new NodeServiceClientCallbackImpl();
    NodeServiceClientCallback remoteCallback = (NodeServiceClientCallback) UnicastRemoteObject.exportObject(callback, 0);
    
    nodeService.uploadTask(NodeServiceTaskType.NS_JAVA_GENERIC_TASK, remoteTask, remoteCallback);*/
    
    uploadTask(NS_JAVA_GENERIC_TASK, task);
  }

  /**
   * @return the hostname
   */
  public String getHostname() {
    return host;
  }

  /**
   * @return the port
   */
  public int getPort() {
    return port;
  }

  public synchronized void gcTask(String taskID) throws RemoteException {
    nodeService.gcTaskResources(taskID);
  }

  public synchronized void invokeJavaPersistentTask(NSJavaPersistentTaskInvocation task) throws RemoteException {
    uploadTask(NS_JAVA_PERSISTENT_TASK_INVOCATION, task);
  }

  public synchronized void uploadJavaPersistentTask(NSJavaPersistentTaskUpload task) throws RemoteException {
    uploadTask(NS_JAVA_PERSISTENT_TASK_UPLOAD, task);
  }
  
  private synchronized <TaskType extends NSTaskRemote> void uploadTask (NodeServiceTaskType type, TaskType task) throws RemoteException {
    if (task == null) {
      throw new IllegalArgumentException("Provided task is null.");
    }
    
    TaskType remoteTask = (TaskType) UnicastRemoteObject.exportObject(task, 0);

    NodeServiceClientCallbackImpl callback = new NodeServiceClientCallbackImpl();
    NodeServiceClientCallback remoteCallback = (NodeServiceClientCallback) UnicastRemoteObject.exportObject(callback, 0);
    
   remoteTask.changeActionState(NSTaskActionState.SCHEDULED, "Task with ID " + remoteTask.getTaskID() + " scheduled for Node Service on host " + host + " and port " + port);
    nodeService.uploadTask(type, remoteTask, remoteCallback);
  }

  public void removePersistentTask(String keyIdentifier, boolean forcedMode, boolean synchronous) throws RemoteException {
    
    if (keyIdentifier == null) {
      throw new IllegalArgumentException("Provided task identifier is null");
    }
    
    nodeService.removePersistentTask(keyIdentifier, forcedMode, synchronous);
  }

  public void startPersistentTask(String keyIdentifier) throws RemoteException {

    if (keyIdentifier == null) {
      throw new IllegalArgumentException("Provided task identifier is null");
    }
    
    nodeService.startPersistentTask(keyIdentifier);
  }

  public void stopJavaTask(String taskID, boolean synchronous) throws RemoteException {

    if (taskID == null) {
      throw new IllegalArgumentException("Provided task identifier is null");
    }
    
    nodeService.stopJavaTask(taskID, synchronous);
  }

  public void stopPersistentTask(String keyIdentifier, boolean forcedMode, boolean synchronous) throws RemoteException {

    if (keyIdentifier == null) {
      throw new IllegalArgumentException("Provided task identifier is null");
    }
    
    nodeService.stopPersistentTask(keyIdentifier, forcedMode, synchronous);
  }

  public void close() {
    nodeService = null;
    NSAPIFactory.removeNodeServiceClient(host, port);
  }
  
  
}
