
package bg.unisofia.fmi.grosd.ns.api.task.impl;

import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;

import bg.unisofia.fmi.grosd.ns.api.util.NodeServiceAPIUtilities;
import bg.unisofia.fmi.grosd.ns.remote.task.NSJavaPersistentTaskInvocation;
import bg.unisofia.fmi.grosd.ns.remote.task.arg.FileArgument;
import bg.unisofia.fmi.grosd.ns.remote.task.arg.NSTaskArgument;
import bg.unisofia.fmi.grosd.ns.remote.task.method.NSTaskMethodPrototype;
import bg.unisofia.fmi.grosd.ns.remote.task.result.locator.NSFileResultLocator;


/**
 * @see NSJavaPersistentTaskInvocation
 *@author Shenol Yousouf
 */
public class NSJavaPersistentTaskInvocationImpl extends NSAbstractTask implements NSJavaPersistentTaskInvocation {

  private String keyIdentifier;
  private FileArgument[] files;
  private NSFileResultLocator locator;
  private NSTaskArgument[]  args;
  private NSTaskMethodPrototype method;
  private String[] initClassnames;
  private NSTaskArgument[] initArgs;
  
  NSJavaPersistentTaskInvocationImpl (String keyIdentifier, NSTaskArgument[] args, FileArgument[] files,  NSTaskMethodPrototype method, String[] initClassnames, NSTaskArgument[] initArgs) {
    super();
    this.keyIdentifier = keyIdentifier;
    this.args = args;
    this.files = files;
    this.method = method;
    this.initClassnames = initClassnames;
    this.initArgs = initArgs;
  }
  
  NSJavaPersistentTaskInvocationImpl (String keyIdentifier, NSTaskArgument[] args, FileArgument[] files, NSTaskMethodPrototype method) {
    this(keyIdentifier, args, files, method, null, null);
  }
  
  NSJavaPersistentTaskInvocationImpl (String keyIdentifier, NSTaskArgument[] args, FileArgument[] files, String[] initClassnames, NSTaskArgument[] initArgs) {
    this(keyIdentifier, args, files, null, initClassnames, initArgs);
  }
  
  NSJavaPersistentTaskInvocationImpl (String keyIdentifier, NSTaskArgument[] args, FileArgument[] files) {
    this(keyIdentifier, args, files, null, null, null);
  }
  
  NSJavaPersistentTaskInvocationImpl (String keyIdentifier, NSTaskArgument[] args) {
    this(keyIdentifier, args, new FileArgument[] {});
  }
  
  public void addFileResultLocator(NSFileResultLocator locator) throws RemoteException {
    this.locator = locator;
  }

  public FileArgument[] getAdditionalFiles() throws RemoteException {
    files = NodeServiceAPIUtilities.getArgsAsRemote(files);
    return files;
  }

  public NSTaskArgument[] getArguments() throws RemoteException {
    args = NodeServiceAPIUtilities.getArgsAsRemote(args);
    return args;
  }

  public NSFileResultLocator getFileResultLocator() throws RemoteException {
    if (locator != null && locator.isExported() == false) {
      locator = (NSFileResultLocator) UnicastRemoteObject.exportObject(locator, 0);
      locator.setExported();
    }
    return locator;
  }

  public NSTaskMethodPrototype getMethodPrototype() throws RemoteException {
    if (method != null && method.isExported() == false) {
      method = (NSTaskMethodPrototype) UnicastRemoteObject.exportObject(method, 0);
      method.setExported();
    }
    return method;
  }

  public String getKeyIdentifier() throws RemoteException {
    return keyIdentifier;
  }

  public NSTaskArgument[] getInitArgs() throws RemoteException {
    return initArgs;
  }

  public String[] getInitParameterClassnames() throws RemoteException {
    return initClassnames;
  }

}
