
package bg.unisofia.fmi.grosd.ns.api.task.impl;

import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;

import bg.unisofia.fmi.grosd.ns.api.util.NodeServiceAPIUtilities;
import bg.unisofia.fmi.grosd.ns.enums.NSTaskActionState;
import bg.unisofia.fmi.grosd.ns.remote.task.NSJavaPersistentTaskUpload;
import bg.unisofia.fmi.grosd.ns.remote.task.arg.FileArgument;
import bg.unisofia.fmi.grosd.ns.remote.task.method.NSTaskMethodPrototype;
import bg.unisofia.fmi.grosd.ns.remote.task.targetclass.NSTaskTargetClass;


/**
 * @see NSJavaPersistentTaskUpload
 *@author Shenol Yousouf
 */
public class NSJavaPersistentTaskUploadImpl extends NSAbstractTask implements NSJavaPersistentTaskUpload {
  
  private String keyIdentifier;
  private NSTaskTargetClass targetClass;
  private FileArgument[] binaries;
  private NSTaskMethodPrototype defaultMethod;
  
  NSJavaPersistentTaskUploadImpl(String keyIdentifier, NSTaskTargetClass targetClass, FileArgument[] binaries, NSTaskMethodPrototype defaultMethod) {
    super();
    this.keyIdentifier = keyIdentifier;
    this.targetClass = targetClass;
    this.binaries = binaries;
    this.defaultMethod = defaultMethod;
  }
  
  NSJavaPersistentTaskUploadImpl(String keyIdentifier, NSTaskTargetClass targetClass, FileArgument[] binaries) {
    this(keyIdentifier, targetClass, binaries, null);
  }

  public FileArgument[] getBinaryFiles() throws RemoteException {
    this.binaries = NodeServiceAPIUtilities.getArgsAsRemote(binaries);
    return binaries;
  }

  public NSTaskMethodPrototype getDefaultMethodPrototype() throws RemoteException {
    
    if (defaultMethod != null && defaultMethod.isExported() == false) {
      defaultMethod = (NSTaskMethodPrototype) UnicastRemoteObject.exportObject(defaultMethod, 0);
      defaultMethod.setExported();
    }
    return defaultMethod;
  }

  public NSTaskTargetClass getTargetClass() throws RemoteException {
    if (this.targetClass.isExported() == false) {
      this.targetClass = (NSTaskTargetClass) UnicastRemoteObject.exportObject(this.targetClass, 0);
      this.targetClass.setExported();
    }
    return this.targetClass;
  }

  public String getKeyIdentifier() throws RemoteException {
    return keyIdentifier;
  }

}
