
package bg.unisofia.fmi.grosd.ns.api.task.impl;

import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;

import bg.unisofia.fmi.grosd.ns.api.util.NodeServiceAPIUtilities;
import bg.unisofia.fmi.grosd.ns.remote.task.NSJavaTask;
import bg.unisofia.fmi.grosd.ns.remote.task.arg.FileArgument;
import bg.unisofia.fmi.grosd.ns.remote.task.method.NSTaskMethod;
import bg.unisofia.fmi.grosd.ns.remote.task.result.locator.NSFileResultLocator;
import bg.unisofia.fmi.grosd.ns.remote.task.targetclass.NSTaskTargetClass;


/**
 * @see NSJavaTask
 *@author Shenol Yousouf
 */
public class NSJavaTaskImpl extends NSAbstractTask implements NSJavaTask {
  
  private NSTaskTargetClass targetClass;
  private NSTaskMethod taskMethod;
  private FileArgument[] binaries;
  private FileArgument[] additionalFiles;
  private NSFileResultLocator locator;
  
  NSJavaTaskImpl ( NSTaskTargetClass targetClass, NSTaskMethod taskMethod, FileArgument[] binaries, FileArgument[] additionalFiles ) {
    super();
    this.targetClass = targetClass;
    this.taskMethod = taskMethod;
    this.binaries = binaries;
    this.additionalFiles = additionalFiles;
  }
  
  NSJavaTaskImpl( NSTaskTargetClass targetClass, NSTaskMethod taskMethod, FileArgument[] binaries ) {
    this( targetClass, taskMethod, binaries, new FileArgument[] {});
  }
  
  NSJavaTaskImpl( NSTaskTargetClass targetClass, NSTaskMethod taskMethod ) {
    this( targetClass, taskMethod, new FileArgument[] {});
  }

  public FileArgument[] getAdditionalFiles() throws RemoteException {
    additionalFiles = NodeServiceAPIUtilities.getArgsAsRemote(additionalFiles);
    return additionalFiles;
  }

  public FileArgument[] getBinaryFiles() throws RemoteException {
    this.binaries = NodeServiceAPIUtilities.getArgsAsRemote(binaries);
    return binaries;
  }

  public NSTaskMethod getMethod() throws RemoteException {
    if (this.taskMethod.isExported() == false) {
      this.taskMethod = (NSTaskMethod) UnicastRemoteObject.exportObject(taskMethod, 0);
      this.taskMethod.setExported();
    }
    return this.taskMethod;
  }

  public NSTaskTargetClass getTargetClass() throws RemoteException {
    if (this.targetClass.isExported() == false) {
      this.targetClass = (NSTaskTargetClass) UnicastRemoteObject.exportObject(this.targetClass, 0);
      this.targetClass.setExported();
    }
    return targetClass;
  }
  
  /* (non-Javadoc)
   * @see bg.unisofia.fmi.grosd.ns.remote.task.NSTaskRemote#addFileResultLocator(bg.unisofia.fmi.grosd.ns.remote.task.result.locator.NSFileResultLocator)
   */
  public void addFileResultLocator(NSFileResultLocator locator) throws RemoteException {
    this.locator = locator;
  }

  /* (non-Javadoc)
   * @see bg.unisofia.fmi.grosd.ns.remote.task.NSTaskRemote#getFileResultLocator()
   */
  public NSFileResultLocator getFileResultLocator() throws RemoteException {
    if (locator != null && locator.isExported() == false) {
      locator = (NSFileResultLocator) UnicastRemoteObject.exportObject(locator, 0);
      locator.setExported();
    }
    return locator;
  }
  

/*  public Remote exportInternally() throws RemoteException {
    try {
      UnicastRemoteObject.unexportObject(this, true);
    } catch (Exception e) {}
    NSJavaTask remote = (NSJavaTask) UnicastRemoteObject.exportObject(this, 0);
    this.additionalFiles = NodeServiceAPIUtilities.getArgsAsRemote(additionalFiles);
    this.binaries = NodeServiceAPIUtilities.getArgsAsRemote(binaries);
    this.taskMethod = (NSTaskMethod) taskMethod.exportInternally();
    this.targetClass = (NSTaskTargetClass) targetClass.exportInternally();
    return remote;
  }*/

}
