
package bg.unisofia.fmi.grosd.ns.api.task.method.impl;

import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;

import bg.unisofia.fmi.grosd.ns.remote.task.arg.NSTaskArgument;
import bg.unisofia.fmi.grosd.ns.remote.task.method.NSTaskMethod;
import bg.unisofia.fmi.grosd.ns.remote.task.method.NSTaskMethodPrototype;


/**
 * Factory for creating <code>NSTaskMethod</code> and <code>NSTaskMethodPrototype</code> instances
 * from various combinations of input parameters.
 *@author Shenol Yousouf
 */
public abstract class NSTaskMethodFactory {

  /**
   * @param methodName method name
   * @param parameterClassnames list of input argument class names
   * @param args input <code>NSTaskArgument</code> instances.
   * @return a <code>NSTaskMethod</code> instance.
   */
  public static NSTaskMethod createNSTaskMethod(String methodName, String[] parameterClassnames, NSTaskArgument... args) {
    return new NSTaskMethodImpl(methodName, parameterClassnames, args);
  }
  
  /**
   * @param methodName method name
   * @param args input <code>NSTaskArgument</code> instances; they are used directly for obtaining the input patameters classnames
   * @return a <code>NSTaskMethod</code> instance
   */
  public static NSTaskMethod createNSTaskMethod(String methodName, NSTaskArgument... args) {
    return new NSTaskMethodImpl(methodName, args);
  }
  
  /**
   * @param methodName method name
   * @return a <code>NSTaskMethod</code> instance which takes no arguments
   */
  public static NSTaskMethod createNSTaskMethod(String methodName)  {
    return new NSTaskMethodImpl(methodName);
  }
  
  /**
   * @param methodName method name
   * @param parameterClassnames list of input argument class names
   * @return a <code>NSTaskMethodPrototype</code> instance
   */
  public static NSTaskMethodPrototype createNSTaskMethodPrototype(String methodName, String[] parameterClassnames) {
    return new NSTaskMethodPrototypeImpl(methodName, parameterClassnames);
  }
  
  /**
   * @param methodName method name
   * @return a <code>NSTaskMethodPrototype</code> instance which takes no arguments
   */
  public static NSTaskMethodPrototype createNSTaskMethodPrototype(String methodName) {
    return new NSTaskMethodPrototypeImpl(methodName);
  }
}
