
package bg.unisofia.fmi.grosd.ns.api.task.result.locator;

import bg.unisofia.fmi.grosd.ns.remote.task.result.locator.NSFileFilter;
import bg.unisofia.fmi.grosd.ns.remote.task.result.locator.NSFileResultLocator;


/**
 * Factory for creating <code>NSFileResultLocator</code> instances from various input arguments combinations.
 *@author Shenol Yousouf
 */
public abstract class NSFileResultLocatorFactory {

  /**
   * @param folderNames folder names for search
   * @param fileFilter file filter for identifying the required files
   * @param searchInSubfolders if <code>true</code>, the search also includes all subfolders recursively
   * @return a <code>NSFileResultLocator</code> instance
   */
  public static NSFileResultLocator createNSFileResultLocator(String[] folderNames, NSFileFilter fileFilter, boolean searchInSubfolders) {
    return new NSFileResultLocatorImpl(folderNames, fileFilter, searchInSubfolders);
  }
  
  /**
   * @param folderNames folder names for search
   * @param fileFilter file filter for identifying the required files
   * @return a <code>NSFileResultLocator</code> instance
   */
  public static NSFileResultLocator createNSFileResultLocator(String[] folderNames, NSFileFilter fileFilter) {
    return new NSFileResultLocatorImpl(folderNames, fileFilter);
  }
  
  /**
   * @param fileFilter file filter for identifying the required files
   * @return a <code>NSFileResultLocator</code> instance for searching inside the Node Service working folder
   */
  public static NSFileResultLocator createNSFileResultLocator(NSFileFilter fileFilter) {
    return new NSFileResultLocatorImpl(fileFilter);
  }
}
