
package bg.unisofia.fmi.grosd.ns.api.task.targetclasses.impl;

import java.io.File;
import java.net.MalformedURLException;
import java.net.URL;
import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;

import bg.unisofia.fmi.grosd.ns.remote.task.arg.NSTaskArgument;
import bg.unisofia.fmi.grosd.ns.remote.task.targetclass.NSTaskTargetClass;


/**
 * Factory for creating <code>NSTaskTargetClass</code> instances from various input combinations.
 *@author Shenol Yousouf
 */
public abstract class NSTaskTargetClassFactory {

  /**
   * @param className name of the target class for executing the task 
   * @param initArgs list of <code>NSTaskArgument</code> instances required for the initialization of a class instance used for the method invocation;
   * their classes are used for identifying the required constructor
   * @return a <code>NSTaskTargetClass</code> instance
   */
  public static NSTaskTargetClass createNSTaskTargetClass (String className,  NSTaskArgument[] initArgs) {
    return new NSTaskTargetClassImpl(className, initArgs);
  }
  
  /**
   * @param className name of the target class for executing the task
   * @param argClassnames list of class names for identifying the constructor for the initialization of a class instance used for the method invocation 
   * @param initArgs list of <code>NSTaskArgument</code> instances required for the initialization of a class instance used for the method invocation
   * @return a <code>NSTaskTargetClass</code> instance
   */
  public static NSTaskTargetClass createNSTaskTargetClass (String className, String[] argClassnames,  NSTaskArgument[] initArgs) {
    return new NSTaskTargetClassImpl(className,argClassnames,  initArgs);
  }
  
}
