
package bg.unisofia.fmi.grosd.ns.api.util;

import java.rmi.RemoteException;
import java.rmi.server.UnicastRemoteObject;

import bg.unisofia.fmi.grosd.ns.remote.task.arg.FileArgument;
import bg.unisofia.fmi.grosd.ns.remote.task.arg.NSTaskArgument;
import bg.unisofia.fmi.grosd.ns.remote.task.arg.SerializableArgument;
import bg.unisofia.fmi.grosd.ns.remote.task.arg.visitor.NSTaskArgumentVisitor;


/**
 * General-purpose class, used for some small side tasks.
 *@author Shenol Yousouf
 */
public abstract class NodeServiceAPIUtilities {
  
  /**
   * @return a task ID which has to be unique
   */
  public static String generateTaskID() {
    return "NS_" + "_" + System.currentTimeMillis();        // current implementation of task ID generation
  }
  
  public static NSTaskArgument[] getArgsAsRemote(NSTaskArgument[] args) throws RemoteException {
    
    class NSTaskArgumentConvertor implements NSTaskArgumentVisitor {
      
      private NSTaskArgument remote;

      public void visit(SerializableArgument arg) throws RemoteException {
        if (arg.isExported()) {
          remote = arg;
        } else {
          remote = (SerializableArgument) UnicastRemoteObject.exportObject(arg, 0);
          remote.setExported();
        }
      }

      public void visit(FileArgument arg) throws RemoteException {
        if (arg.isExported()) {
          remote = arg;
        } else {
          remote = (FileArgument) UnicastRemoteObject.exportObject(arg, 0);
          arg.setExported();
        }
      }
      
      public NSTaskArgument getRemoteArgument() {
        return remote;
      }
    }
    
    if (args == null) {
      return null;
    }
    
    NSTaskArgument[] remote = new NSTaskArgument[args.length];
    int index = 0;
    NSTaskArgumentConvertor convertor = new NSTaskArgumentConvertor();
    for(NSTaskArgument arg : args) {
     arg.accept(convertor);
     remote[index] = convertor.getRemoteArgument();
      index++;
    }
    return remote;
  }
  
  public static FileArgument[] getArgsAsRemote(FileArgument[] args) throws RemoteException {
    if (args == null) {
      return null;
    }
    
    FileArgument[] remote = new FileArgument[args.length];
    int index = 0;
    for (FileArgument arg : args) {
      if (arg.isExported()) {
        remote[index] = arg;
      } else {
        remote[index] = (FileArgument) UnicastRemoteObject.exportObject(arg, 0);
        remote[index].setExported();
      }
      index++;
    }
    return remote;
  }
}
