
package bg.unisofia.fmi.grosd.ns.remote;

import java.rmi.RemoteException;


/**
 * The interface is needed for the custom model of exporting Remote objects
 * for the components of NodeService framework. Most components contain fields
 * that are Remote objects too and has to be exported AFTER the export of
 * the containing object itself; if this is done before the export, there are problems
 * with their location in the rmiregistry table.
 * 
 * The main idea is that the components to which the custom exporting rules has to be applied,
 * are exported at the moment when they are needed, but only ONCE. Hence, the need for 
 * an exporting flag and the corresponding accessors declared in the interface. After the first export,
 * the exported remote objects are "cached" for the next attempts for access from the remote side. 
 * 
 *@author Shenol Yousouf
 */
public interface NodeServiceExportableObject {

  public boolean isExported() throws RemoteException;
  public void setExported() throws RemoteException;
}
