
package bg.unisofia.fmi.grosd.ns.remote.task;

import java.rmi.RemoteException;

import bg.unisofia.fmi.grosd.ns.remote.task.arg.FileArgument;
import bg.unisofia.fmi.grosd.ns.remote.task.arg.NSTaskArgument;
import bg.unisofia.fmi.grosd.ns.remote.task.method.NSTaskMethodPrototype;
import bg.unisofia.fmi.grosd.ns.remote.task.result.locator.NSFileResultLocator;


/**
 * Task interface for invoking a persistent task already uploaded on the called node.
 * Defines the set of metadata necessary for the actual invocation.
 *@author Shenol Yousouf
 */
public interface NSJavaPersistentTaskInvocation extends NSTaskRemote {

  /**
   * Add a helper object for locating the files required by NS API client, generated as a result from the invocation.
   * @param locator helpe robject for locating the files required by NS API client, generated as a result from the invocation
   * @throws RemoteException
   */
  public void addFileResultLocator(NSFileResultLocator locator) throws RemoteException;
  
  /**
   * @return get the added helper object for locating the files required by NS API client, generated as a result from the invocation
   * @throws RemoteException
   */
  public NSFileResultLocator getFileResultLocator() throws RemoteException;
  
  /**
   * @return string that uniquely identifies the persistent task in the Node Service;
   * subsequent invocations of the persistent task should refer to this key for the task identification;
   * two persistent tasks with the same key identifiers are not allowed on the same node.
   * @throws RemoteException
   */
  public String getKeyIdentifier() throws RemoteException;
  
  /**
   * @return actual arguments for invocation
   */
  public NSTaskArgument[] getArguments() throws RemoteException;

  /**
   * @return list of class names for initialization of target class instance for this specific invocation
   * @throws RemoteException
   */
  public String[] getInitParameterClassnames() throws RemoteException;
  
  /**
   * @return  list of constructor arguments for initialization of target class instance for this specific invocation
   * @throws RemoteException
   */
  public NSTaskArgument[] getInitArgs() throws RemoteException;  
  
  /**
   * @return actual method for invocation; it overrides the default method if such one is defined
   */
  public NSTaskMethodPrototype getMethodPrototype() throws RemoteException;
  
  /**
   * @return additional files that are resources for the task execution, for example configuration and input files
   * @throws RemoteException
   */
  public FileArgument[] getAdditionalFiles() throws RemoteException;
}
