
package bg.unisofia.fmi.grosd.ns.remote.task;

import java.rmi.RemoteException;

import bg.unisofia.fmi.grosd.ns.remote.task.arg.FileArgument;
import bg.unisofia.fmi.grosd.ns.remote.task.method.NSTaskMethodPrototype;
import bg.unisofia.fmi.grosd.ns.remote.task.targetclass.NSTaskTargetClass;


/**
  * Task interface for uploading a persistent task to the called node.
 *  Defines the set of metadata necessary for the actual upload.
 *@author Shenol Yousouf
 */
public interface NSJavaPersistentTaskUpload extends NSTaskRemote {

  /**
   * @return string that uniquely identifies the persistent task in the Node Service;
   * subsequent invocations of the persistent task should refer to this key for the task identification;
   * two persistent tasks with the same key identifiers are not allowed on the same node.
   * @throws RemoteException
   */
  public String getKeyIdentifier() throws RemoteException;
  
  /**
   * @return target class for invoking method calls on; note that this target class instance is assumed to contain
   * the parameters for default initialzation but during the invocation call it is allowed to change the used constructor parameters;
   * actually, this target class instance is required to contain only a valid class name for the target class -
   * the initialization parameters can be omitted entirely so that they will be specified at each invocation call
   * @throws RemoteException
   */
  public NSTaskTargetClass getTargetClass() throws RemoteException;
  
  /**
   * @return default method for invocation; can be null, which means that the method for invocation will be specified on each call
   * @throws RemoteException
   */
  public NSTaskMethodPrototype getDefaultMethodPrototype() throws RemoteException;
  
  /**
   * @return .class and .jar files for classloading
   * @throws RemoteException
   */
  public FileArgument[] getBinaryFiles() throws RemoteException;
}
