
package bg.unisofia.fmi.grosd.ns.remote.task;

import java.rmi.RemoteException;

import bg.unisofia.fmi.grosd.ns.remote.task.arg.FileArgument;
import bg.unisofia.fmi.grosd.ns.remote.task.method.NSTaskMethod;
import bg.unisofia.fmi.grosd.ns.remote.task.result.locator.NSFileResultLocator;
import bg.unisofia.fmi.grosd.ns.remote.task.targetclass.NSTaskTargetClass;


/**
  * Task interface that defines the set of metadata necessary for invoking a method of a Java program..
 *@author Shenol Yousouf
 */
public interface NSJavaTask extends NSTaskRemote {

  /**
   * Add a helper object for locating the files required by NS API client, generated as a result from the invocation.
   * @param locator helpe robject for locating the files required by NS API client, generated as a result from the invocation
   * @throws RemoteException
   */
  public void addFileResultLocator(NSFileResultLocator locator) throws RemoteException;
  
  /**
   * @return get the added helper object for locating the files required by NS API client, generated as a result from the invocation
   * @throws RemoteException
   */
  public NSFileResultLocator getFileResultLocator() throws RemoteException;
  
  /**
   * @return the metadata about the class which is target of the task
   * @throws RemoteException
   */
  public NSTaskTargetClass getTargetClass() throws RemoteException;
  
  /**
   * @return the metadata about the method which is target of the task
   * @throws RemoteException
   */
  public NSTaskMethod getMethod() throws RemoteException;
  
  /**
   * @return .class and .jar files for classloading
   * @throws RemoteException
   */
  public FileArgument[] getBinaryFiles() throws RemoteException;
  
  /**
   * @return additional files that are resources for the task execution, for example configuration or input files;
   * note that these file should not be returned by this method if they are already packaged with the binary files
   * @throws RemoteException
   */
  public FileArgument[] getAdditionalFiles() throws RemoteException; 
}
