
package bg.unisofia.fmi.grosd.ns.remote.task;

import java.rmi.Remote;
import java.rmi.RemoteException;
import java.util.List;

import bg.unisofia.fmi.grosd.ns.enums.NSTaskActionState;
import bg.unisofia.fmi.grosd.ns.remote.listener.NSTaskListener;


/**
 * The root interface for all kinds of tasks that Node Service officially supports.
 * This interface should  never be directly implemented by a non-abstract class.
 * It can be only extended by the subinterfaces of the actual task types.
 * The interface declares the minimum functionality that every task type should support. 
 *@author Shenol Yousouf
 */
public interface NSTaskRemote extends Remote {

  /**
   * @return the action state of task during execution
   * @throws RemoteException
   */
  public NSTaskActionState getActionState() throws RemoteException;
  
  /**
   * Add listener object for monitoring the task execution.
   * @param listener object for monitoring the task execution
   * @throws RemoteException
   */
  public void addListener(NSTaskListener listener) throws RemoteException;
  
  /**
   * @return all registered listeners for this task
   * @throws RemoteException
   */
  public List<NSTaskListener> getListeners()  throws RemoteException;

  /**
   * @return ID for the task; this ID is unique at least on a node level but according to the requirements of the Node Service API users,
   * it could be unique in a wider area, for example on a cluster or on a grid level
   * @throws RemoteException
   */
  public String getTaskID() throws RemoteException;  
  
  /**
   * Change the action state of task when its execution goes into a new phase.
   * @param state   new task state
   * @param comment comment for the reasons for entering a new state
   * @throws RemoteException
   */
  public void changeActionState (NSTaskActionState state, String comment) throws RemoteException;
}
