
package bg.unisofia.fmi.grosd.ns.remote.task.arg;

import java.net.URL;
import java.rmi.RemoteException;


/**
 * A subtype of <code>bg.unisofia.fmi.grosd.ns.remote.task.arg.NSTaskArgument</code> which wraps metadata about file arguments.
 * Files require special treatment as they cannot be transferred directly between hosts as <code>java.io.Serializable</code> object;
 * instead, they have to be copied. NOTE that if your task has a file argument which does not involve an actual file transfer
 * (for example, this is a file to be created during task execution), such argument should be wrapped as <code>bg.unisofia.fmi.grosd.ns.remote.task.arg.SerializableArgument</code>.
 *@author Shenol Yousouf
 */
public interface FileArgument extends NSTaskArgument {

  /**
   * @return get actual URL to the file from where it has to be tranferred
   * @throws RemoteException
   */
  public URL getSourceFileURL()  throws RemoteException;
  
  /**
   * @return destination which is relative to the application location; if null, file is copied to Node Servcie work folder on node host
   * @throws RemoteException
   */
  public String getRelativeDest()  throws RemoteException;
  
  /**
   * @return    if file is accessible only as a local file or as a shared net resource;
   * this defines if the file can be directly transferred to the node or the help of <code>bg.unisofia.fmi.grosd.ns.remote.NodeServiceClientCallback</code> has to be used.
   * @throws RemoteException
   */
  public boolean isLocal() throws RemoteException;
}
