
package bg.unisofia.fmi.grosd.ns.remote.task.arg;

import java.io.Serializable;
import java.rmi.RemoteException;


/**
 * A subtype of <code>bg.unisofia.fmi.grosd.ns.remote.task.arg.NSTaskArgument</code> which wraps metadata about <code>java.io.Serializable</code> arguments.
 * Such arguments can be directly transferred as arguments of remote calls in RMI.
 * However, note that although <code>java.io.File</code> is also a Serializble subtype, its instances are special case for transfer
 * and for this purpose, a separate <code>bg.unisofia.fmi.grosd.ns.remote.task.arg.FileArgument</code> interface is defined.
 *@author Shenol Yousouf
 */
public interface SerializableArgument extends NSTaskArgument {

  /**
   * @return formal input parameter class name; can return <code>null</code> and in this case the class is obtained from actual parameter value;
   * however, you may wish to point out explicitly the class name of the argument to avoid ambiguities coming from
   * class hierarchies or the autoboxing feature in JDK5 or higher - e.g. in certain cases, the <code>int</code> arguments are automatically wrapped
   * in <code>java.lang.Integer</code> objects; Node Service provides mechanisms for type resolution even in these cases
   * but this feature is a performance hit if used too often. 
   * @throws RemoteException
   */
  public String getFormalParameterClassname()  throws RemoteException;
  
  /**
   * @return paramter's actual value as a <code>java.io.Serializable</code> object
   * @throws RemoteException
   */
  public Serializable getValue() throws RemoteException;
}
