
package bg.unisofia.fmi.grosd.ns.remote.task.result.locator;

import java.io.Serializable;
import java.rmi.Remote;
import java.rmi.RemoteException;

import bg.unisofia.fmi.grosd.ns.remote.NodeServiceExportableObject;


/**
 * Interface for locating the output files from executing a task on node.
 * The interface is entirely implemented by the clients of the NS API.
 *@author Shenol Yousouf
 */
public interface NSFileResultLocator extends Remote, Serializable, NodeServiceExportableObject {

  /**
   * @return folder names that are searched for output files from task execution 
   * @throws RemoteException
   */
  public String[] getFolderNamesForSearch() throws RemoteException;
  
  /**
   * @return the custom file filter for identifying the output files
   * @throws RemoteException
   */
  public NSFileFilter getFileFilter() throws RemoteException;
  
  /**
   * @return whether to extend the search into the subfolder of the already defined folders recursively;
   * this can substantially increase the time for locating the output files in case of a complex directory structure 
   * @throws RemoteException
   */
  public boolean searchInSubfolders() throws RemoteException;
}
