
package bg.unisofia.fmi.grosd.ns.remote.task.targetclass;

import java.io.Serializable;
import java.rmi.Remote;
import java.rmi.RemoteException;

import bg.unisofia.fmi.grosd.ns.remote.NodeServiceExportableObject;
import bg.unisofia.fmi.grosd.ns.remote.task.arg.NSTaskArgument;


/**
 * Interface for obtaining metadata about the class which is target of task execution.
 *@author Shenol Yousouf
 */
public interface NSTaskTargetClass extends Remote, Serializable, NodeServiceExportableObject {

  /**
   * @return name of the target class
   * @throws RemoteException
   */
  public String getClassName() throws RemoteException;
  
  /**
   * @return class names of parameters for identifying the constructor for instantiating an object of this class
   * which later is used for invoking a method for executing the actual task; can return <code>null</code> or
   * an empty list, because these class names can be also obtained from provided parameter values;
   *  however, in such cases, the same considerations about type resolution,
   * such as with defining <code>SerializableArgument</code> instances, apply here as well;
   * @see bg.unisofia.fmi.grosd.ns.remote.task.arg.SerializableArgument#getFormalParameterClassname()
   * @throws RemoteException
   */
  public String[] getParameterClassnames() throws RemoteException;
  
  /**
   * @return list of parameter values for  instantiating an object of this class
   * which later is used for invoking a method for executing the actual task
   * @throws RemoteException
   */
  public NSTaskArgument[] getInitArgs() throws RemoteException;
}
