
package bg.unisofia.fmi.grosd.ns.remote.util.impl;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.rmi.RemoteException;

import bg.unisofia.fmi.grosd.ns.remote.util.NSOutputStreamWrapper;


/**
 * Uniform implementation of <code> bg.unisofia.fmi.grosd.ns.remote.util.NSOutputStreamWrapper</code>, used
 * both by the Node Service API and Node Service kernel.
 * @see NSOutputStreamWrapper
 *@author Shenol Yousouf
 */
public class NSOutputStreamWrapperImpl implements NSOutputStreamWrapper {
  
  private BufferedOutputStream wrapped;
  private URL localURL;
  
  public NSOutputStreamWrapperImpl(File file) throws IOException {
    this.wrapped = new BufferedOutputStream(new FileOutputStream(file));
    this.localURL = file.toURI().toURL();
  }

  public void close() throws RemoteException {
    try {
      this.wrapped.close();
    } catch (IOException e) {
      throw new RemoteException("Error closing remote output stream", e);
    }
  }

  public void write(byte[] b, int length) throws RemoteException {
    try {
      this.wrapped.write(b, 0, length);
    } catch (IOException e) {
      throw new RemoteException("Error writing to remote output stream", e);
    }
  }

  public URL getLocalOutputURL() throws RemoteException {
    return localURL;
  }

}
